/*
 *
 *   This file is part of BingDownloader.
 *
 *   BingDownloader is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   BingDownloader is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with BingDownloader.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   (c) 2012 Sebastian Bota (sebastian.bota@live.com)
 */

#include "bingdownloadermainwindow.h"
#include "ui_bingdownloadermainwindow.h"

BingDownloaderMainWindow::BingDownloaderMainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::BingDownloaderMainWindow)
{
    ui->setupUi(this);
    ui->imageLabel->setBackgroundRole(QPalette::Base);
    ui->imageLabel->setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
    ui->imageLabel->setScaledContents(true);

    ui->scrollArea->setBackgroundRole(QPalette::Dark);
    ui->scrollArea->setWidgetResizable(true);

    qBingHelper = new QBingHelper(0);

    imageHistory = qBingHelper->getImageHistory ();
    currentIndex = 0;

    loadImageAndDetails ();
}

BingDownloaderMainWindow::~BingDownloaderMainWindow()
{
    delete ui;
}

void BingDownloaderMainWindow::loadImageAndDetails()
{
    QFile* tempFile = saveImage (true);
    QImage img(tempFile->fileName ());

    ui->imageLabel->setPixmap (QPixmap::fromImage (img));


    QBingImageData imageData = imageHistory.at(currentIndex);
    ui->lblDescription->setText (imageData.getImageDescription());
}

QFile* BingDownloaderMainWindow::saveImage(bool isTemporaryFile)
{
    QBingImageData imageData = imageHistory.at(currentIndex);
    QString imagePath(QDir::tempPath ().append (QDir::separator ()).append (TEMP_FILE_NAME));
    imagePath = QDir::toNativeSeparators (imagePath);

    QFile *bingImageFile = new QFile(imagePath);
    if(!isTemporaryFile) {
        QString imageFileName = QFileDialog::getSaveFileName (this,tr("Save picture"), QDir::currentPath().append(imageData.getImageName()));
        QFile::copy (imagePath,imageFileName);
        bingImageFile = new QFile(imageFileName);
    }else {
        QNetworkAccessManager *networkMgr = new QNetworkAccessManager(this);
        QNetworkReply *reply = networkMgr->get( QNetworkRequest( QUrl(imageData.getImageURL()) ) );
        QEventLoop loop;
        QObject::connect(reply, SIGNAL(finished()), &loop, SLOT(quit()));
        loop.exec();

        if (!bingImageFile->open(QIODevice::WriteOnly)) {
            QMessageBox::information(this, tr("BingDownloader"),
                                     tr("Unable to save the file %1: %2.")
                                     .arg(imagePath).arg(bingImageFile->errorString()));
            delete bingImageFile;
            bingImageFile = 0;
        }
        bingImageFile->write(reply->readAll());
        bingImageFile->flush();
        bingImageFile->close ();
        bingImageFile->deleteLater();
    }

    return bingImageFile;
}

void BingDownloaderMainWindow::on_actionNext_Image_triggered()
{
    if(currentIndex == 0)
    {
        currentIndex = imageHistory.size ();
    }
    currentIndex--;
    loadImageAndDetails ();

}

void BingDownloaderMainWindow::on_actionPrevious_Image_triggered()
{
    currentIndex++;
    if(currentIndex == imageHistory.size()) {
        currentIndex = 0;
    }
    loadImageAndDetails ();
}

void BingDownloaderMainWindow::on_actionSave_Image_triggered()
{
    saveImage (false);
}

void BingDownloaderMainWindow::on_actionToday_image_triggered()
{
    currentIndex = 0;
    loadImageAndDetails();
}

void BingDownloaderMainWindow::on_actionAbout_triggered()
{
    About *about = new About(this);
    about->setWindowModality (Qt::WindowModal);
    about->show ();
}
