/*
 *
 *   This file is part of BingDownloader.
 *
 *   BingDownloader is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   BingDownloader is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with BingDownloader.  If not, see <http://www.gnu.org/licenses/>.
 *
 *   (c) 2012 Sebastian Bota (sebastian.bota@live.com)
 */

#include "qbinghelper.h"

QBingHelper::QBingHelper( QObject * parent ) : QObject(parent)
{
    pageSourceCode = getWebPageSource ( BING_URL);
}
QString QBingHelper::getImageUrlInternal(QString pageContent, QString queryString)
{
    QString stringToFind(queryString);
    int startIndex = pageContent.indexOf (stringToFind);
    startIndex += stringToFind.length();
    int lastIndex = pageContent.indexOf('\'', startIndex);

    QString imagePath = pageContent.mid(startIndex, lastIndex - startIndex);
    imagePath = imagePath.replace ("\\", "");
    return imagePath;
}

/*
 * Retrieves the following details:
 *   - image url
 *   - image description
 */
QBingImageData QBingHelper::getTodayBingImageWithDescription()
{
    QString currentImageUrl = getImageUrlInternal (pageSourceCode,TODAY_IMG_PREFIX);
    QString imageURL = QString(BING_URL).append (currentImageUrl);
    QString tmp(TODAY_IMAGE_DESC);
    int startIndex = pageSourceCode.indexOf (tmp);
    startIndex+=tmp.length ();
    int lastIndex = pageSourceCode.indexOf('\'', startIndex);

    QString description = pageSourceCode.mid(startIndex, lastIndex - startIndex);
    description = description.mid (0,description.indexOf ("</p>"));

    return QBingImageData(imageURL,description);
}
/*
  Get image history from bing

*/
QList<QBingImageData> QBingHelper::getImageHistory()
{
   QList<QBingImageData> result;
    //add the image from today !
    QBingImageData todayImageData = getTodayBingImageWithDescription();
    result.append(todayImageData);

    QString bingHistoryURL (HISTORY_URL);
    QString sourceJSon = getWebPageSource (bingHistoryURL);

    QStringList src = sourceJSon.split (HISTORY_URL_PREFIX);

    src.removeAt (0); // first does not count some leftover from JSon

    foreach(QString obj ,src)
    {
        QString copyrightTextToFind(HISTORY_IMG_DESC_START_PREFIX);
        int startIndex = obj.indexOf (copyrightTextToFind);

        QString url = obj.mid(0,obj.indexOf("\""));
        startIndex +=copyrightTextToFind.length();
        int lastIndex = obj.indexOf ("\",\"copyrightlink\"") -startIndex;
        QString description = obj.mid (startIndex,lastIndex);
        url = url.replace ("\"","");
        description.replace ("\"","");


        result.append(QBingImageData(QString(BING_URL).append (url),description));
    }

    return result;

}

QString QBingHelper::getWebPageSource(QString query)
{
    QNetworkAccessManager *networkMgr = new QNetworkAccessManager(this);
    QNetworkReply *reply = networkMgr->get( QNetworkRequest( QUrl(query) ) );
    QEventLoop loop;
    QObject::connect(reply, SIGNAL(finished()), &loop, SLOT(quit()));
    loop.exec();
    return reply->readAll();

}
